/*
 * Copyright (c) 2002 MackyBee <macky@email.plala.or.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package pharerouge.utils;

import java.io.UnsupportedEncodingException;
import java.text.BreakIterator;
import java.util.Locale;
import java.util.ResourceBundle;
/*import java.nio.charset.Charset;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.CoderResult;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;*/
import java.io.*;

/**
 * This class has encoding related static util methods and keeps some default 
 * system properties related them into static variables.
 * @since 2002.11.11
 * @author MackyBee
 */
public final class EncodingUtil {

    /* 2005.10.30 Gardien removed. "Default encoding" is now what the user chooses
	rather than the system's file encoding.
	If you want to know the system's file encoding, please use directly System.getProperty.
	//Keeps default encoding (file.encoding) properly obtained.
    // 2002.12.19 MackyBee modified. 
    // This must be called before this property is initially updated.
    public static String defaultEncoding;
        //= System.getProperty(
        //"file.encoding");

    // 2002.12.19 MackyBee added experimentally. 
    static {
        try {
            defaultEncoding = System.getProperty("file.encoding");
            if (defaultEncoding == null) {
                throw new Exception("Exception: file.encoding is null!");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
 */
    
    // 2002.11.29 MackyBee added these 2 lines.
    final static char COMBINING_VOICED_SOUND_MARK = '\u3099';
    final static char COMBINING_SEMI_VOICED_SOUND_MARK = '\u309A';
    
    /**
     * For before Java 1.4 version. 
     */
    // 2002.12.19 MackyBee experimentally added. 
    static String[] encodingList;
    
    private EncodingUtil() {}

    /**
     * Gets encoding list available on this application for before Java 1.4 
     * versions. <br>
     * This method makes original encoding list. 
     */
    // 2002.12.19 MackyBee added. 

    public static String[] getAvailableEncodingList() {
        if (encodingList == null) {
		// 2005.10.30 Gardien modified. Asks the java language a complete list of encodings.
		Object[] tempObjects = java.nio.charset.Charset.availableCharsets().keySet().toArray();
		encodingList = new String[tempObjects.length];
		for (int i=0; i<tempObjects.length; i++)	{
			encodingList[i] = tempObjects[i].toString();
		}
/*		encodingList = new String[] {
			defaultEncoding, 
	                "ISO-8859-1", 
	                "UTF-16", 
	                "Shift_JIS"
		};*/
        }
        return (encodingList);
    }

    /**
     * Under a circumstance, String.getBytes(String) can not return properly 
     * value in spite of certain charset name argument. But this workaround 
     * method could solve that problem.
     * @return If inner defaultEncoding value is incorrect, returns 0 length 
     * byte array reference value. But that case, it must be VM's bug.
     */
    public final static byte[] strictGetBytes(String s) {
        try {
            //{{{ deactivated process. 
            // Temporary
            /*
            byte[] defbytes = s.getBytes();
            System.out.println("EncodingUtil.strictGetBytes "
                + "(default value): Source [" + s + "] ");
            for (int b = 0; b < defbytes.length; b++) {
                System.out.print(" " + String.valueOf(defbytes[b]));
            }
            System.out.println();
            
            byte[] modbytes = new String(s.getBytes(
                EncodingUtil.DEFAULT_ENCODING)).getBytes();
             System.out.println("EncodingUtil.strictGetBytes "
                + "(modified value): Source [" + s + "] ");
            for (int b = 0; b < modbytes.length; b++) {
                System.out.print(" " + String.valueOf(modbytes[b]));
            }
            System.out.println();
            */
            //}}}
            
            return (new String(s.getBytes(System.getProperty("file.encoding"))).
                getBytes());
        } catch (UnsupportedEncodingException e) {
            return (new byte[0]);
        }
    }
    
    // 2002.11.11 MackyBee experimentally added.
    public final static String strictGetString(byte[] bytes) {
        try {
            // 2002.11.13 MackyBee modified.
            return (new String(bytes, System.getProperty("file.encoding")));
        } catch (UnsupportedEncodingException e) {
            return ("");
        }
    }
    
    // 2002.11.13 MackyBee added.
    public final static String strictGetString(byte[] bytes, int offset, 
        int length) {
        try {
            // 2002.11.13 MackyBee modified.
            return (new String(bytes, offset, length, 
                System.getProperty("file.encoding")));
        } catch (UnsupportedEncodingException e) {
            return ("");
        }
    }
    
    /** 
     * This method fixes a Japanese code compatibility issue in Unicode. 
     */
    // 2002.11.29 MackyBee added.
    public static String fixJapaneseDynamicPunctuationString(String s) {
        
        StringBuffer converted = new StringBuffer();
        
        BreakIterator boundary = BreakIterator.getCharacterInstance(
            Locale.JAPANESE);
        boundary.setText(s);
        
        int start = boundary.first();
        for (int end = boundary.next(); end != BreakIterator.DONE;
            start = end, end = boundary.next()) {
            String c = s.substring(start, end);
            if (c.length() == 2) {
                converted.append(convertDynamicPunctuationCharacter(c));
            } else {
                converted.append(c);
            }
        }
        return (converted.toString());
    }
    
    /** This method is used by fixJapaneseDynamicPunctuationString. */
    // 2002.11.29 MackyBee added.
    static String convertDynamicPunctuationCharacter(String s) {
        char voicedSoundMark = s.charAt(1);
        switch (voicedSoundMark) {
            case COMBINING_VOICED_SOUND_MARK: 
                return (new Character((char) (s.charAt(0) + '\u0001')).
                    toString());
            case COMBINING_SEMI_VOICED_SOUND_MARK: 
                return (new Character((char) (s.charAt(0) + '\u0002')).
                    toString());
            default: 
                return (s);
        }
    }


	// 2005.10.30 Gardien added.
	// Interprets a resource bundle parameter with the correct encoding of the locale file.
	public static String convert(ResourceBundle resource, String parameter) {
		return convert(resource.getString(parameter), resource.getString("_localeFileEncoding"));
	}

	// 2006.01.24 Gardien added.
	// Converts a string to a specific encoding.
	public static String convert(String someString, String encoding) {
		String convertedString = null;
		try {
			// ISO-8859-1 so far seems to be the only locale that reads bytes correctly.
			convertedString = new String(someString.getBytes("iso-8859-1"), encoding);
		} catch (Exception e) {;}  // Was not able to convert
		if (convertedString != null) {
			return convertedString;
		} else {
			return someString;	// Was not able to convert
		}
	}
}

